# Generated by Django 3.2.12 on 2022-11-15 16:46

from django.db import migrations, models
import django_countries.fields
import phonenumber_field.modelfields


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0002_auto_20221115_0817'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='address',
            options={'verbose_name': 'Address'},
        ),
        migrations.RemoveField(
            model_name='generalinfo',
            name='phone',
        ),
        migrations.AddField(
            model_name='address',
            name='address_line_1',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AddField(
            model_name='address',
            name='address_line_2',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AddField(
            model_name='address',
            name='city',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AddField(
            model_name='address',
            name='country',
            field=django_countries.fields.CountryField(blank=True, max_length=2, null=True),
        ),
        migrations.AddField(
            model_name='address',
            name='postal_code',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name='Post/Zip-code'),
        ),
        migrations.AddField(
            model_name='address',
            name='state',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='State/Province'),
        ),
        migrations.AddField(
            model_name='generalinfo',
            name='email',
            field=models.EmailField(blank=True, max_length=254, null=True),
        ),
        migrations.AddField(
            model_name='generalinfo',
            name='phone_number',
            field=phonenumber_field.modelfields.PhoneNumberField(blank=True, max_length=128, region=None),
        ),
        migrations.AddField(
            model_name='generalinfo',
            name='website',
            field=models.URLField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.2.12 on 2022-10-06 16:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('profiles', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='job_title',
            field=models.CharField(blank=True, max_length=75),
        ),
    ]




from django.conf import settings
from django.conf.urls import include, url
from django.contrib import admin
from django.urls import path, include



urlpatterns = [
   path('django-admin/', admin.site.urls),
   path(r'', include("multitenancy.urls")),
   url(r"^admin/teams/", include("pinax.teams.urls", namespace="pinax_teams")),
   url(r"^admin/invitations/", include("pinax.invitations.urls", namespace="pinax_invitations")),
   path("select2/", include("django_select2.urls")),
   path('admin/support/', include('helpdesk.urls', namespace='helpdesk')),
   url(r"^admin/account/", include("account.urls")),
   url(r'^admin/settings/groups-manager/', include('groups_manager.urls', namespace='groups_manager')),
    
]
if settings.DEBUG:
   from django.conf.urls.static import static
   from django.contrib.staticfiles.urls import staticfiles_urlpatterns
   urlpatterns += staticfiles_urlpatterns()
   urlpatterns+= static(settings.MEDIA_URL,document_root=settings.MEDIA_ROOT)+static(settings.STATIC_URL,document_root=settings.STATIC_ROOT)




# Generated by Django 3.2.12 on 2023-01-07 15:42

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0006_auto_20230107_1742'),
        ('apps', '0011_auto_20230101_0650'),
    ]

    operations = [
        migrations.RenameField(
            model_name='domain',
            old_name='has_custom',
            new_name='is_custom',
        ),
        migrations.AddField(
            model_name='tenant',
            name='subscription',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='tenants', to='subscriptions.subscription'),
        ),
        migrations.AddField(
            model_name='tenant',
            name='trail_duration',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='tenant',
            name='plan',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='tenants', to='subscriptions.plan'),
        ),
    ]



"""django-multitenancy-manager setup"""
from distutils.util import convert_path
from fnmatch import fnmatchcase
import os
from setuptools import find_packages, setup
import sys
from multitenancy import __version__

version = __version__


# Provided as an attribute, so you can append to these instead
# of replicating them:
standard_exclude = ("*.py", "*.pyc", "*$py.class", "*~", ".*", "*.bak", ".gcloudignore", "*.yaml", "")
standard_exclude_directories = (
    ".*",
    "CVS",
 
    ".vscode"
    "./env",
    "_darcs",
    "./build",
    "./dist",
    "EGG-INFO",
    "*.egg-info",
)

# (c) 2005 Ian Bicking and contributors; written for Paste (http://pythonpaste.org)
# Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php
# Note: you may want to copy this into your setup.py file verbatim, as
# you can't import this from another package, when you don't know if
# that package is installed yet.


def find_package_data(
    where=".*",
    package=["multitenancy", "multitenancy/project_template" ],
    exclude=standard_exclude,
    exclude_directories=standard_exclude_directories,
    only_in_packages=True,
    show_ignored=False,
):
    """
    Return a dictionary suitable for use in ``package_data``
    in a distutils ``setup.py`` file.

    The dictionary looks like::

        {'package': [files]}

    Where ``files`` is a list of all the files in that package that
    don't match anything in ``exclude``.

    If ``only_in_packages`` is true, then top-level directories that
    are not packages won't be included (but directories under packages
    will).

    Directories matching any pattern in ``exclude_directories`` will
    be ignored; by default directories with leading ``.``, ``CVS``,
    and ``_darcs`` will be ignored.

    If ``show_ignored`` is true, then all the files that aren't
    included in package data are shown on stderr (for debugging
    purposes).

    Note patterns use wildcards, or can be exact paths (including
    leading ``./``), and all searching is case-insensitive.
    """

    out = {}
    stack = [(convert_path(where), "", package, only_in_packages)]
    while stack:
        where, prefix, package, only_in_packages = stack.pop(0)
        for name in os.listdir(where):
            fn = os.path.join(where, name)
            if os.path.isdir(fn):
                bad_name = False
                for pattern in exclude_directories:
                    if fnmatchcase(name, pattern) or fn.lower() == pattern.lower():
                        bad_name = True
                        if show_ignored:
                            print(
                                "Directory %s ignored by pattern %s" % (
                                    fn, pattern),
                                file=sys.stderr,
                            )

                        break
                if bad_name:
                    continue
                if os.path.isfile(os.path.join(fn, "__init__.py")) and not prefix:
                    if not package:
                        new_package = name
                    else:
                        new_package = package + "." + name
                    stack.append((fn, "", new_package, False))
                else:
                    stack.append((fn, prefix + name + "/",
                                  package, only_in_packages))
            elif package or not only_in_packages:
                # is a file
                bad_name = False
                for pattern in exclude:
                    if fnmatchcase(name, pattern) or fn.lower() == pattern.lower():
                        bad_name = True
                        if show_ignored:
                            print(
                                "File %s ignored by pattern %s" % (
                                    fn, pattern),
                                file=sys.stderr,
                            )
                        break
                if bad_name:
                    continue
                out.setdefault(package, []).append(prefix + name)
    return out


def get_requirements():
    with open(os.path.join(os.path.dirname(__file__), "requirements.txt")) as f:
        requirements_list = [req.strip() for req in f.readlines()]

    requirements_list.append("setuptools")
    requirements_list.append("pytz")
    return requirements_list


def get_long_description():
    with open(os.path.join(os.path.dirname(__file__), "README.md")) as f:
        long_desc = f.read()

    return long_desc

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


setup(
    name="django-multitenancy-manager",
    version=version,
    description="Django Multitenancy is a Django app that handles user subscriptions for tenant apps.",
    long_description=get_long_description(),
    long_description_content_type='text/markdown',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Environment :: Web Environment",
        "Operating System :: OS Independent",
        "Intended Audience :: Customer Service",
        "License :: OSI Approved :: BSD License",
        "Topic :: Software Development :: Bug Tracking",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Office/Business",
        "Natural Language :: English",
    ],
    keywords=[
        "django",
        "django-tenants",
        "multitenancy",
        "django-multitenancy",
        'multitenants',
        "django-tenant-users",
        "saas",
        "django-saas",
        "track",
        "support",
    ],
    author="Tekano Khambane",
    author_email="tkhambane@gmail.com",
    maintainer="Tekano Khambane",
    maintainer_email="tkhambane@gmail.com",
    url="https://github.com/tekanokhambane/django-multitenancy",
    license="BSD",
    #packages=["multitenancy"],
    packages=find_packages(),
 #   package_data=find_package_data("multitenancy", only_in_packages=False),
    include_package_data=True,
    zip_safe=False,
    install_requires=get_requirements(),
    entry_points={
        "console_scripts": [
            "multitenancy=multitenancy.bin.multitenancy:main"
        ]
    },
    # entry_points="""
    #     [console_scripts]
    #     multitenancy=multitenancy.bin.multitenancy:main
    # """,
    
)

#python setup.py sdist
#python setup.py bdist_wheel
#twine upload --repository pypi dist/*



from django.shortcuts import render
from account.mixins import LoginRequiredMixin
from multitenancy.admin.views.baseViews import AdminTemplateView


class OrderIndexView(AdminTemplateView):
    template_name = "multitenancy/order/order_index.html"


class OrdersListView(LoginRequiredMixin ,AdminTemplateView):

    template_name = 'multitenancy/order/orders_list.html'


from django.urls import path
from . import views
urlpatterns = [
    path('billing/',views.BillingIndexView.as_view(), name="billing"),
    path('billing/invoices/',views.InvoiceListView.as_view(), name="invoice_list"),
    path('billing/refunds/',views.RefundsListView.as_view(), name="refunds_list"),
    path('billing/credits/',views.CreditsListView.as_view(), name="credits_list"),
]


import os
from django.conf import settings

from account.mixins import LoginRequiredMixin
from multitenancy.admin.views.baseViews import CustomerTemplateView
from multitenancy.apps.models import Tenant, Domain
from account.models import Account
from helpdesk.models import Ticket



class CustomerIndexView(LoginRequiredMixin, CustomerTemplateView):
    template_name = "multitenancy/admin/publicUser/index.html"
    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        context['subscriptions'] = Tenant.objects.filter(owner=self.request.user)
        context['domains'] = Domain.objects.filter(tenant__owner=self.request.user).filter(is_custom=True)
        context['account'] = Account.objects.get_or_create(user=self.request.user)
        context['tickets'] = Ticket.objects.filter(
            submitter_email=self.request.user.email, status=1)
        context['alltickets'] = Ticket.objects.filter(submitter_email=self.request.user.email)
        
        return context
    





import unittest
from django.test import RequestFactory, TestCase, Client
from multitenancy.admin.views.adminViews import (
    AdminIndexView, 
    )
from multitenancy.users.views import(
    CreateCustomerView,
    CreateStaffView,
    DeleteCustomerView,
    CustomerListView,
    UpdateCustomerView,
    UpdateStaffView,
)
from multitenancy.subscriptions.models import Plan
from multitenancy.users.models import Admin, Customer, Staff, TenantUser
from  multitenancy.admin.decorators import allowed_users
from multitenancy.users.forms import CustomerForm, CustomerUpdateForm
from multitenancy.subscriptions.forms import PlanForm
from multitenancy.admin import urls



class CustomerViewsTestCase(unittest.TestCase):

    def setUp(self):
        self.factory = RequestFactory()
        self.client = Client()
        self.customer_data = {
            'first_name': 'customer',
            'last_name': 'test',
            'username': 'tcustomer',
            'password': 'password1234',
            'email': 'customer@example.com',
        }
        self.new_customer_data = {
            'first_name': 'customer1',
            'username': 'testcustomer',
        }
        self.customer_obj = Customer.objects.first()


    def test_get_create_customer_view(self):
        self.user = TenantUser.objects.get(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/customers/create/')
        request.user = self.user
        response = CreateCustomerView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(self.client.session.get('_auth_user_id'))
        self.assertIn('_auth_user_id', self.client.session)

                                              
    def test_post_create_customer_view(self):
        self.user = TenantUser.objects.get(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        self.client.force_login(user=self.user)
        request = self.factory.post('/admin/customers/create/', data=self.customer_data)
        request.user = self.user
        response = CreateCustomerView.as_view()(request)
        self.assertEqual(response.status_code, 200)

    def test_get_create_customer_view_authenticated(self):
        self.user = TenantUser.objects.get(
            
            email='AnonymousUser', 
            
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/customers/create/')
        request.user = self.user
        response = AdminIndexView.as_view()(request)
        
        self.assertEqual(response.status_code, 404)
    
    def test_update_customer_view(self):
        self.user = TenantUser.objects.get(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        self.customer = Customer.objects.create(
            username='customer', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='customer123@email.com', 
        )
        request = self.factory.post(f'/admin/customers/{self.customer.id}/update', data=self.new_customer_data)
        request.user = self.user
        response = UpdateCustomerView.as_view()(request, pk=self.customer.id)
        self.assertEqual(response.status_code, 200)

    
    def test_update_customer_unauthenticateduser_view(self):
        self.user = TenantUser.objects.get(
            email='AnonymousUser', 
            )
        
        self.client.force_login(user=self.user)
        self.customer = Customer.objects.create(
            username='customer', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='customer123e@email.com', 
        )
        request = self.factory.post(f'/admin/customers/{self.customer.id}/update', data=self.new_customer_data)
        request.user = self.user
        response = UpdateCustomerView.as_view()(request, pk=self.customer.id)
        self.assertEqual(response.status_code, 404)


    def test_update_customer_authenticatednonadminuser_view(self):
        self.user = TenantUser.objects.create(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='staff', 
            email='staff1@email.com', 
            type='Staff',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        self.customer = Customer.objects.create(
            username='customer', 
            password="password", 
            first_name='abc123', 
            last_name='customer1', 
            email='customer12344@email.com', 
        )
        request = self.factory.post(f'/admin/customers/{self.customer.id}/update', data=self.new_customer_data)
        request.user = self.user
        response = UpdateCustomerView.as_view()(request, pk=self.customer.id)
        self.assertEqual(response.status_code, 404)

    def test_delete_customer_view(self):
        self.user = TenantUser.objects.get(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        self.customer = Customer.objects.create(
            username='customer', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='customer12345@email.com', 
        )
        request = self.factory.delete(f'/admin/customers/{self.customer.id}/delete')
        request.user = self.user
        response = DeleteCustomerView.as_view()(request, pk=self.customer.id)
        self.assertEqual(response.status_code, 302)

    def test_delete_customer_unauthenticateduser_view(self):
        self.user = TenantUser.objects.get(
            email='AnonymousUser', 
            )
        
        self.client.force_login(user=self.user)
        self.customer = Customer.objects.create(
            username='customer', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='customer1234@email.com', 
        )
        request = self.factory.delete(f'/admin/customers/{self.customer.id}/delete')
        request.user = self.user
        response = DeleteCustomerView.as_view()(request, pk=self.customer.id)
        self.assertEqual(response.status_code, 404)    
    

    def test_customerlist_index_view(self):
        self.user = TenantUser.objects.get(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/customers/')
        request.user = self.user
        response = CustomerListView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(self.client.session.get('_auth_user_id'))
        self.assertIn('_auth_user_id', self.client.session)

    def test_customerlist_index_view_authenticated(self):
        self.user = TenantUser.objects.get(
            
            email='AnonymousUser', 
            
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/customers/')
        request.user = self.user
        response = CustomerListView.as_view()(request)
        
        self.assertEqual(response.status_code, 404)

class StaffViewsTestCase(unittest.TestCase):

    def setUp(self):
        self.factory = RequestFactory()
        self.client = Client()
        self.staff_data = {
            'first_name': 'staff',
            'last_name': 'test',
            'username': 'tstaff',
            'password': 'password1234',
            'email': 'staff@example.com',
        }
        self.new_staff_data = {
            'first_name': 'staff1',
            'username': 'teststaff',
        }
        self.staff_obj = Staff.objects.first()

    def test_get_create_staff_view(self):
        self.user = TenantUser.objects.get(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/staff/create/')
        request.user = self.user
        response = CreateCustomerView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(self.client.session.get('_auth_user_id'))
        self.assertIn('_auth_user_id', self.client.session)

                                              
    def test_post_create_staff_view(self):
        self.user = TenantUser.objects.get(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        self.client.force_login(user=self.user)
        request = self.factory.post('/admin/staff/create/', data=self.staff_data)
        request.user = self.user
        response = CreateStaffView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        

    def test_get_create_staff_view_authenticated(self):
        self.user = TenantUser.objects.get(
            email='AnonymousUser', 
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/staff/create/')
        request.user = self.user
        response = AdminIndexView.as_view()(request)
        
        self.assertEqual(response.status_code, 404)
    
    def test_update_staff_view(self):
        self.user = TenantUser.objects.get(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        self.staff = Staff.objects.create(
            username='staff2', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='staff123@email.com', 
        )
        request = self.factory.post(f'/admin/custom/{self.staff.id}/update', data=self.new_staff_data)
        request.user = self.user
        response = UpdateStaffView.as_view()(request, pk=self.staff.id)
        self.assertEqual(response.status_code, 200)

    
    def test_update_customer_unauthenticateduser_view(self):
        self.user = TenantUser.objects.get(
            email='AnonymousUser', 
            )
        
        self.client.force_login(user=self.user)
        self.staff = Staff.objects.create(
            username='staff', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='staff123e@email.com', 
        )
        request = self.factory.post(f'/admin/staff/{self.staff.id}/update/', data=self.new_staff_data)
        request.user = self.user
        response = UpdateStaffView.as_view()(request, pk=self.staff.id)
        self.assertEqual(response.status_code, 404)


    def test_delete_customer_view(self):
        self.user = TenantUser.objects.get(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        self.customer = Customer.objects.create(
            username='customer', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='customer12345@email.com', 
        )
        request = self.factory.delete(f'/admin/customers/{self.customer.id}/delete')
        request.user = self.user
        response = DeleteCustomerView.as_view()(request, pk=self.customer.id)
        self.assertEqual(response.status_code, 302)

    def test_delete_customer_unauthenticateduser_view(self):
        self.user = TenantUser.objects.get(
            email='AnonymousUser', 
            )
        
        self.client.force_login(user=self.user)
        self.customer = Customer.objects.get(
            username='customer', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='customer1234@email.com', 
        )
        request = self.factory.delete(f'/admin/customers/{self.customer.id}/delete')
        request.user = self.user
        response = DeleteCustomerView.as_view()(request, pk=self.customer.id)
        self.assertEqual(response.status_code, 404)   


from django.conf import settings
from django.urls import include, path, re_path
from django.contrib import admin


urlpatterns = [
    
]


if settings.DEBUG:
    from django.conf.urls.static import static
    from django.contrib.staticfiles.urls import staticfiles_urlpatterns

    # Serve static and media files from development server
    urlpatterns += staticfiles_urlpatterns()
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)


from django.conf import settings
from django.db import models, transaction

from django.db.models import Q, Sum
from django_tenants.models import DomainMixin
from datetime import datetime, timedelta
from tenant_users.tenants.models import TenantBase
from django_tenants.utils import get_tenant_type_choices

from multitenancy.subscriptions.models import Plan, Subscription


DEFAULT_TYPE = "personal"

def get_types():
    #remove the plublic tenant type from list
    items = get_tenant_type_choices()
    items.pop(0) 
    return items

class TenantQuerySet(models.QuerySet):
    def search(self, query=None):
        if query is None or query =="":
            return self.all()
        lookups = Q(name__icontains=query ) | Q(id__contains=query) | Q(type__iexact=query)| Q(owner__id__icontains=query)| Q(owner__email__icontains=query)| Q(owner__username__icontains=query)
        return self.filter(lookups)
    
    def filter_by_plan(self, plan):
        filter = self.filter(type=plan)
        return filter

    def active(self):
        return self.filter(subscription__is_active=True)
    
    

class TenantManager(models.Manager):
    def get_queryset(self):
        return TenantQuerySet(self.model, using=self._db)
    
    def search(self, query=None):
        return self.get_queryset().search(query=query)
    
    def filter_by_plan(self, plan):
        return self.get_queryset().filter_by_plan(plan=plan)
    
    def active(self):
        return self.get_queryset().active()

class Tenant(TenantBase):
    id = models.AutoField(primary_key=True, auto_created=True)
    type = models.CharField(max_length=200, default=DEFAULT_TYPE, choices=get_types())
    name = models.CharField(max_length=100)
    is_template = models.BooleanField(default=True)
    plan = models.ForeignKey(Plan, null=True, on_delete=models.PROTECT, related_name="tenants")
    description = models.TextField(max_length=200)
    subscription = models.OneToOneField(Subscription,  blank=True,on_delete=models.CASCADE, related_name="tenants")
    # paid_until = models.DateField()
    trail_duration = models.IntegerField(default=0)
    on_trial = models.BooleanField(default=False)
    created = models.DateField(auto_now_add=True)
    modified = models.DateField(auto_now=True)
    objects = TenantManager()

    def __str__(self):
        return self.name

    class Meta:
        unique_together = ['id', 'name']
        verbose_name = settings.TENANT_DISPLAY_NAME
        verbose_name_plural = settings.TENANT_DISPLAY_NAME_PLURAL
    

    @classmethod
    def create_tenant(cls, name, type, is_template, description, trail_duration, owner, subscription_plan):
        subscription = Subscription.objects.create(plan=subscription_plan, owner=owner)
        tenant = cls.objects.create(name=name, type=type, is_template=is_template, description=description, trail_duration=trail_duration, subscription=subscription, owner=owner)
        return tenant
    
    @transaction.atomic
    def create_tenant_with_subscription(self, plan, **kwargs):
        subscription = Subscription.objects.create(plan=plan)
        kwargs['subscription'] = subscription
        tenant = Tenant.objects.create(**kwargs)
        return tenant
    
    @classmethod
    def total_revenue(cls):
        revenue = cls.objects.aggregate(Sum('plan__price'))['plan__price__sum']
        if revenue is None:
            return 0
        return revenue

    def start_trail(self):
        """
        if the user has no active or inactive tenants than start trail
        """
        
        if self.owner.tenants.count() > 2:
            self.on_trial = False
            self.trail_duration = 0
            self.subscription.start_subscription("monthly")
            self.save()
        else:
            self.on_trial = True
            self.trail_duration = 30
            self.subscription.start_subscription("monthly")
            self.save()

    def end_trail(self):
        """
        End trail when the trail duration is reached
        """
        if self.on_trial == True:
            self.on_trial = False
            self.trail_duration = None
            self.deactivate()
            self.save()            
    

    
    def trail_days_left(self):
        """
        Calculate the number of days left for the trail to end
        """
        if self.on_trial:
            trail_end_date = self.created + timedelta(days=self.trail_duration)
            days_left = (trail_end_date - datetime.now().date()).days
            # update the duration
            self.trail_duration = days_left
            self.save()
            # update duration and end trail then deactivate service
            if self.trail_duration == 0:
                self.on_trial = False
                self.deactivate()
                self.save()
            return days_left
        else:
            return None

    def upgrade(self, plan):
        """
        Iterate over the plans and then select a plan to upgrade and update the subscription
        """
        self.type = plan
        self.save()

    def downgrade(self):
        """
        Downgrade plan
        """
        pass
    
    def add_subscription(self):
        """
        if user upgrades service from trail create a subscription
        """
        if self.on_trial == True:
            subscription = self.subscription.objects.create()
            self.subscription.get_product_type("tenant")
            self.save()
        
    



class DomainManager(models.Manager):
    pass

class Domain(DomainMixin):
    is_custom = models.BooleanField(default=False)
    objects = DomainManager()

    def __str__(self):
        return self.domain


# Generated by Django 3.2.12 on 2022-10-17 13:13

from django.db import migrations, models
import django_tenants.postgresql_backend.base


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Domain',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('domain', models.CharField(db_index=True, max_length=253, unique=True)),
                ('is_primary', models.BooleanField(db_index=True, default=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DomainTld',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(choices=[('.co.za', '.co.za'), ('.com', '.com'), ('.org', '.org'), ('.org.za', '.org.za'), ('.net', '.net'), ('.africa', '.africa'), ('.web.za', '.web.za'), ('.net.za', '.net.za'), ('.joburg', '.joburg'), ('.capetown', '.capetown'), ('.durban', '.durban'), ('.za.com', '.za.com'), ('.sa.com', '.sa.com'), ('.mobi', '.mobi'), ('.info', '.info'), ('.biz', '.biz'), ('.name', '.name'), ('.co', '.co'), ('.cc', '.cc'), ('.co.uk', '.co.uk'), ('.org.uk', '.org.uk'), ('.gr', '.gr'), ('.me', '.me'), ('.mx', '.mx'), ('.us', '.us'), ('.tv', '.tv'), ('.vc', '.vc'), ('.ws', '.ws'), ('.academy', '.academy'), ('.agency', '.agency'), ('.art', '.art'), ('.bargains', '.bargains'), ('.bike', '.bike'), ('.build', '.build'), ('.builders', '.builders'), ('.camera', '.camera'), ('.camp', '.camp'), ('.careers', '.careers'), ('.center', '.center'), ('.city', '.city'), ('.clothing', '.clothing'), ('.cloud', '.cloud'), ('.club', '.club'), ('.codes', '.codes'), ('.coffee', '.coffee'), ('.company', '.company'), ('.computer', '.computer'), ('.construction', '.construction'), ('.consulting', '.consulting'), ('.contractors', '.contractors'), ('.diamonds', '.diamonds'), ('.directory', '.directory'), ('.design', '.design'), ('.domains', '.domains'), ('.education', '.education'), ('.email', '.email'), ('.energy', '.energy'), ('.engineering', '.engineering'), ('.enterprises', '.enterprises'), ('.enterprises', '.enterprises'), ('.equipment', '.equipment'), ('.estate', '.estate'), ('.farm', '.farm'), ('.florist', '.florist'), ('.fund', '.fund'), ('.gallery', '.gallery'), ('.glass', '.glass'), ('.global', '.global'), ('.graphics', '.graphics'), ('.green', '.green'), ('.guru', '.guru'), ('.healthcare', '.healthcare'), ('.holiday', '.holiday'), ('.one', '.one'), ('.online', '.online'), ('.marketing', '.marketing'), ('.ninja', '.ninja'), ('.rocks', '.rocks'), ('.run', '.run'), ('.services', '.services'), ('.solutions', '.solutions'), ('.space', '.space'), ('.store', '.store'), ('.tips', '.tips'), ('.tech', '.tech'), ('.trade', '.trade'), ('.university', '.university')], max_length=30, null=True)),
                ('price', models.DecimalField(decimal_places=2, max_digits=5)),
                ('transfer_price', models.DecimalField(decimal_places=2, default=50, max_digits=5)),
                ('period', models.IntegerField(default=1)),
            ],
        ),
        migrations.CreateModel(
            name='Packages',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(choices=[('public', 'public'), ('personal', 'personal'), ('premium', 'premium'), ('business', 'business')], default='personal', max_length=100)),
                ('price', models.DecimalField(decimal_places=2, default=75, max_digits=12, verbose_name='Price')),
            ],
        ),
        migrations.CreateModel(
            name='RegisteredDomain',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sld', models.CharField(max_length=250, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Tenant',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('schema_name', models.CharField(db_index=True, max_length=63, unique=True, validators=[django_tenants.postgresql_backend.base._check_schema_name])),
                ('slug', models.SlugField(blank=True, verbose_name='Tenant URL Name')),
                ('type', models.CharField(default='personal', max_length=200)),
                ('name', models.CharField(default='name', max_length=100)),
                ('description', models.TextField(max_length=200)),
                ('on_trial', models.BooleanField(default=True)),
                ('created', models.DateField(auto_now_add=True)),
                ('modified', models.DateField(auto_now=True)),
            ],
        ),
    ]



import os
import uuid
from django.db import models
from django.db.models import Q
from tenant_users.tenants.models import UserProfile, UserProfileManager
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import Group
from tenant_users.permissions.models import UserTenantPermissions
from multitenancy.profiles.models import Profile


def upload_avatar_to(instance, filename):
    filename, ext = os.path.splitext(filename)
    return os.path.join(
        'avatar_images',
        'avatar_{uuid}_{filename}{ext}'.format(
            uuid=uuid.uuid4(), filename=filename, ext=ext)
    )


class TenantUserQuerySet(models.QuerySet):
        def search(self, query=None):
            if query is None or query =="":
                return self.all()
            lookups = Q(first_name__icontains=query ) | Q(last_name__icontains=query ) | Q(email__icontains=query) | Q(username__icontains=query)
            return self.filter(lookups)
        
        def filter_by_id(self, query=None):
            if query is None or query =="":
                return self.all()
            return self.filter(id__exact=query)
        
class TenantUserManager(UserProfileManager):
    def get_queryset(self):
        return TenantUserQuerySet(self.model, using=self._db)
    
    def search(self, query=None):
        return self.get_queryset().search(query=query)
    
    def filter_by_id(self, query=None):
        return self.get_queryset().filter_by_id(query=query)
    
    
# Create your models here.
class TenantUser(UserProfile):
    class Types(models.TextChoices):
        ADMIN = "Admin", "Admin"
        STAFF = "Staff", "Staff"
        CUSTOMER = "Customer", "Customer"
    avatar = models.ImageField(
        verbose_name=_('profile picture'),
        upload_to=upload_avatar_to,
        blank=True,
    )

    type = models.CharField(_('Type'), max_length=255, choices=Types.choices, default=Types.CUSTOMER)
    first_name = models.CharField(max_length=300, blank=True, null=True)
    last_name = models.CharField(max_length=300, blank=True, null=True)
    username = models.CharField(max_length=250, blank=True, null=True)
    groups = models.ManyToManyField(Group, blank=True)
    # note = models.CharField(max_length=300, blank=True, null=True)
    signup_confirmation = models.BooleanField(default=False)

    objects = TenantUserManager()

    def __str__(self) -> str:
        if self.first_name and self.last_name:
            return "{} {}".format(self.first_name, self.last_name)
        elif self.first_name:
            return self.first_name
        elif self.last_name:
            return self.last_name
        elif self.username:
            return self.username
        else:
            return self.email

    class Meta:
        verbose_name = _('user profile')
        verbose_name_plural = _('user profiles')

    @property
    def is_superuser(self):
        if self.usertenantpermissions.is_superuser:
            return True
        else:
            return False
    def get_username(self) -> str:
        uname = self.username
        return uname
    
    def add_role(self, role):
        if role not in self.roles:
            self.roles.append(role)
            
    def remove_role(self, role):
        if role in self.roles:
            self.roles.remove(role)

    
    

class AdminManager(models.Manager):
    def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).filter(type=TenantUser.Types.ADMIN)


class StaffManager(models.Manager):
    def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).filter(type=TenantUser.Types.STAFF)


class CustomerManager(models.Manager):
    def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).filter(type=TenantUser.Types.CUSTOMER)


class Admin(TenantUser):
    objects = AdminManager()

    @property
    def get_profile(self):
        profile = Profile.objects.get_or_create(user_id=self.id, name=self.username)  # type: ignore
        return profile

    
    def create_public_superuser(self, *args, **kwargs):
        UserTenantPermissions.objects.create(profile_id=self.pk, is_staff=True, is_superuser=True)

    class Meta:
        proxy = True

    def save(self, *args, **kwargs):
        if not self.pk:
            self.type = TenantUser.Types.ADMIN

        return super().save(*args, **kwargs)


class Staff(TenantUser):
    objects = StaffManager()

    @property
    def profiles(self):
        return self.profiles

    @property
    def account(self):
        return self.account
    
    def add_user_perms(self, *args, **kwargs):
        UserTenantPermissions.objects.create(profile_id=self.pk, is_staff=True, is_superuser=False)

    @property
    def get_profile(self):
        profile = Profile.objects.get_or_create(user_id=self.id, name=self.username)  # type: ignore
        return profile

    class Meta:
        proxy = True

    def save(self, *args, **kwargs):
        if not self.pk:
            self.type = TenantUser.Types.STAFF
        return super().save(*args, **kwargs)


class Customer(TenantUser):
    objects = CustomerManager()

    @property
    def account(self):
        return self.account

    class Meta:
        proxy = True

    def save(self, *args, **kwargs):
        if not self.pk:
            self.type = TenantUser.Types.CUSTOMER
        return super().save(*args, **kwargs)



# Generated by Django 3.2.12 on 2023-02-04 18:29

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('address', '0001_initial'),
        ('subscriptions', '0014_subscription_created_date'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('order', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='order',
            name='shipping_address',
        ),
        migrations.RemoveField(
            model_name='order',
            name='subscription',
        ),
        migrations.AddField(
            model_name='order',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='order',
            name='billing_address',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='address.address'),
        ),
        migrations.CreateModel(
            name='OrderItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='order.order')),
                ('subscription', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='subscriptions.subscription')),
            ],
        ),
    ]



from django.db import models

class Address(models.Model):
    recipient_name = models.CharField(max_length=100)
    street_address = models.CharField(max_length=100)
    city = models.CharField(max_length=50)
    state = models.CharField(max_length=50)
    postal_code = models.CharField(max_length=20)
    country = models.CharField(max_length=50)


# Generated by Django 3.2.12 on 2022-10-06 09:41

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import multitenancy.profiles.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=75)),
                ('avatar', models.ImageField(blank=True, upload_to=multitenancy.profiles.models.avatar_upload)),
                ('bio', models.TextField(blank=True)),
                ('affiliation', models.CharField(blank=True, max_length=100)),
                ('location', models.CharField(blank=True, max_length=100)),
                ('website', models.CharField(blank=True, max_length=250)),
                ('twitter_username', models.CharField(blank=True, max_length=100, verbose_name='Twitter Username')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('modified_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='profile', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Generated by Django 3.2.12 on 2022-10-07 08:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('profiles', '0003_auto_20221007_0951'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='display_profile',
            field=models.BooleanField(default=True),
        ),
    ]



from rest_framework import viewsets
from django.shortcuts import render
from rest_framework.decorators import api_view
from django.http import HttpResponseRedirect
from django.urls.base import reverse, reverse_lazy
from rest_framework.reverse import reverse
from rest_framework import permissions
from account.mixins import LoginRequiredMixin
from multitenancy.admin.views.baseViews import(
    AdminDeleteView,
    AdminCreateView,
    AdminUpdateView,
    AdminTemplateView,
    )
from tenant_users.permissions.models import UserTenantPermissions
from .forms import CustomerForm, CustomerUpdateForm, StaffForm, StaffUpdateForm
from .serializers import UserSerializer
from .models import Customer, TenantUser, Staff


class CustomerListView(LoginRequiredMixin ,AdminTemplateView):
    template_name = "multitenancy/users/customer_list.html"

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        query = self.request.GET.get("q") 
        f = TenantUser.objects.filter(type="Customer").search(query=query)
        context['filter'] = f
        return context

class DeleteCustomerView(LoginRequiredMixin ,AdminDeleteView):
    model = Customer
    template_name = "multitenancy/users/delete_customer.html"
    success_url = reverse_lazy("customer_list", urlconf="multitenancy.urls")

    def delete(self, request, *args, **kwargs):
        customer_id = self.kwargs['pk']

        user = Customer.objects.filter(id=customer_id)
        user.delete()
        return HttpResponseRedirect(reverse('customer_list', urlconf="multitenancy.urls"))

class CreateCustomerView(LoginRequiredMixin ,AdminCreateView):
    model = TenantUser
    form_class = CustomerForm
    # success_url = '/admin/customers/'
    success_url = reverse_lazy('customer_list', urlconf="multitenancy.urls")
    template_name = 'multitenancy/users/create_customer.html'

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['initial'] = {'type': 'Customer'}
        return kwargs

class UpdateCustomerView(LoginRequiredMixin ,AdminUpdateView):
    model = Customer
    form_class = CustomerUpdateForm
    success_url = reverse_lazy('customer_list', urlconf="multitenancy.urls")
    template_name = 'multitenancy/users/update_customer.html'


class StaffListView(LoginRequiredMixin ,AdminTemplateView):
    template_name = "multitenancy/users/staff_list.html"

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        query = self.request.GET.get("q") 
        f = TenantUser.objects.filter(type="Staff").search(query=query)
        context['filter'] = f
        return context
    

class CreateStaffView(LoginRequiredMixin ,AdminCreateView):
    model = TenantUser
    form_class = StaffForm
    # success_url = '/admin/customers/'
    success_url = reverse_lazy('staff_list', urlconf="multitenancy.urls")
    template_name = 'multitenancy/users/create_staff.html'

    def get(self, request, *args, **kwargs):
        form = self.form_class(initial={'type': 'Staff'})
        return render(request, self.template_name, {'form': form})
    
    def form_valid(self, form):
        response = super().form_valid(form)
        user = form.instance
        user_permission = UserTenantPermissions.objects.create(profile_id=user.pk, is_staff=True, is_superuser=False)
        user_permission.save()
        return response

class UpdateStaffView(LoginRequiredMixin ,AdminUpdateView):
    model = Staff
    form_class = StaffUpdateForm
    success_url = reverse_lazy('staff_list', urlconf="multitenancy.urls")
    template_name = 'multitenancy/users/update_staff.html'


class DeleteStaffView(LoginRequiredMixin ,AdminDeleteView):
    model = Staff
    template_name = "multitenancy/users/delete_staff.html"
    success_url = reverse_lazy("staff_list", urlconf="multitenancy.urls")

    def delete(self, request, *args, **kwargs):
        staff_id = self.kwargs['pk']

        user = Staff.objects.filter(id=staff_id)
        user.delete()
        return HttpResponseRedirect(reverse('staff_list', urlconf="multitenancy.urls"))


class CustomerViewSet(viewsets.ModelViewSet):
    """
    This viewset automatically provides `list`, `create`, `retrieve`,
    `update` and `destroy` actions.

    Additionally we also provide an extra `highlight` action.
    """
    model = TenantUser
    
    serializer_class = UserSerializer
    permission_classes = [permissions.IsAuthenticated,
                          permissions.IsAdminUser]

    def get_queryset(self):
        query = self.request.GET.get("q")
        id_query  = self.request.GET.get("id")
        queryset = TenantUser.objects.filter(type="Customer").exclude(email="AnonymousUser").search(query=query).filter_by_id(query=id_query).distinct()
        return queryset


class StaffViewSet(viewsets.ModelViewSet):
    """
    This viewset automatically provides `list`, `create`, `retrieve`,
    `update` and `destroy` actions.

    Additionally we also provide an extra `highlight` action.
    """
    model = TenantUser
    
    serializer_class = UserSerializer
    permission_classes = [permissions.IsAuthenticated,
                          permissions.IsAdminUser]

    def get_queryset(self):
        query = self.request.GET.get("q") 
        queryset = TenantUser.objects.filter(type="Staff").search(query=query)
        return queryset


from rest_framework import serializers
from multitenancy.subscriptions.serializers import  SubscriptionSerializer
from .models import Tenant


class TenantSerializer(serializers.ModelSerializer):
    subscription = SubscriptionSerializer(many=False)
    class Meta:
        model = Tenant
        fields = ['id', 'name', 'type', 'is_template', 'plan', 'description', 'subscription', 'trail_duration', 'on_trial', 'created', 'modified']


from django.urls import path
from . import views

urlpatterns = [
    path("settings/plans/", views.PlanListView.as_view(), name="plan_list"),
    path("settings/plans/<str:slug>/", views.PlanDetailView.as_view(), name="plan_detail"),
    path("settings/plans/feature/create/", views.FeatureCreateView.as_view(), name="create_feature"),
    path("settings/plans/<int:pk>/update/", views.UpdatePlanView.as_view(), name="update_plan"),
    path("settings/plan/create/", views.CreatePlanView.as_view(), name="create_plan"),
    path("settings/plans/<int:pk>/delete/", views.DeletePlanView.as_view(), name="delete_plan"),
    path("settings/usersubscriptions/", views.UserSubscriptionsListView.as_view(), name="usersubscription_list"),
    path("settings/producttypes/", views.ProductTypeListView.as_view(), name="producttype_list"),

]


# Generated by Django 3.2.12 on 2022-10-17 13:13

from django.db import migrations, models
import tenant_users.permissions.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('apps', '0001_initial'),
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='TenantUser',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('email', models.EmailField(db_index=True, max_length=254, unique=True, verbose_name='Email Address')),
                ('is_active', models.BooleanField(default=True, verbose_name='active')),
                ('is_verified', models.BooleanField(default=False, verbose_name='verified')),
                ('type', models.CharField(choices=[('Admin', 'Admin'), ('Staff', 'Staff'), ('Customer', 'Customer')], default='Admin', max_length=255, verbose_name='Type')),
                ('first_name', models.CharField(blank=True, max_length=300, null=True)),
                ('last_name', models.CharField(blank=True, max_length=300, null=True)),
                ('username', models.CharField(blank=True, max_length=250, null=True)),
                ('signup_confirmation', models.BooleanField(default=False)),
                ('groups', models.ManyToManyField(blank=True, to='auth.Group')),
                ('tenants', models.ManyToManyField(blank=True, help_text='The tenants this user belongs to.', related_name='user_set', to='apps.Tenant', verbose_name='tenants')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model, tenant_users.permissions.models.PermissionsMixinFacade),
        ),
        migrations.CreateModel(
            name='Admin',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('users.tenantuser',),
        ),
        migrations.CreateModel(
            name='Customer',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('users.tenantuser',),
        ),
        migrations.CreateModel(
            name='Staff',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('users.tenantuser',),
        ),
    ]



class SettingsConfig:
    def __init__(self, model_class, position, label, icon):
        self.model_class = model_class
        self.position = position
        self.label = label
        self.icon = icon

class Settings:
    app_configs = []

    @classmethod
    def register_settings(cls, model_class, position=0, label=None, icon=None):
        if not label:
            label = model_class._meta.verbose_name
        cls.app_configs.append(SettingsConfig(model_class, position, label, icon))
        return model_class

def add_to_settings(position=0, label=None, icon=None):
    def decorator(cls):
        Settings.register_settings(cls, position, label, icon)
        return cls
    return decorator


class ModelAdminGroup:
    menu_icon = ''
    menu_label = ''
    menu_order = 0
    items = []

    def __init__(self, menu_icon, menu_label, menu_order):
        self.menu_icon = menu_icon
        self.menu_label = menu_label
        self.menu_order = menu_order

    def register(self, model_admin):
        self.items.append(model_admin)



class AppConfig:
    def __init__(self, model_class, menu_label, menu_order, menu_icon, list_display):
        self.model_class = model_class
        self.menu_label = menu_label
        self.menu_order = menu_order
        self.menu_icon = menu_icon
        self.list_display = list_display


class Admin:
    app_configs = []

    def __init__(self, model, menu_label, menu_order, menu_icon, list_display, group, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.model = model
        self.menu_label = menu_label
        self.menu_order = menu_order
        self.menu_icon = menu_icon
        self.list_display = list_display
        group.register(self)
        self.app_configs.append(AppConfig(model, menu_label, menu_order, menu_icon, list_display))


def register_admin(admin_class):
    admin_class.app_configs.append(AppConfig(admin_class.model, admin_class.menu_label, admin_class.menu_order, admin_class.menu_icon, admin_class.list_display))
    return admin_class






from django.urls import path, include
# from django.conf.urls import url
from .admin import team_urls
from .admin import customer_urls
from .admin import urls as admin_urls
from .profiles import urls as profile_urls
from .subscriptions import urls as subscription_urls
from .users import  urls as users_urls
from .apps import urls as apps_urls
from .settings import urls as settings_urls
from .billing import urls as billing_urls
from .order import urls as order_urls
from . import  api


urlpatterns = [
    path('admin/', include(admin_urls)),
    path('admin/', include(users_urls)),
    path('admin/', include(apps_urls)),
    path('admin/', include(subscription_urls)),
    path('admin/', include(billing_urls)),
    path('admin/', include(order_urls)),
    path('admin/', include(settings_urls)),
    path('dashboard/', include(customer_urls)),
    path('admin/', include(team_urls)),
    path('admin/profiles/', include(profile_urls)),
    path('api/', include(api)),
]



from django import forms
from phonenumber_field.formfields import PhoneNumberField
from django_countries.widgets import CountrySelectWidget

from multitenancy.settings.models import Address, AdminSettings, GeneralInfo, Logo

class LogoForm(forms.ModelForm):
    logo = forms.FileField(required=False, widget=forms.FileInput(attrs={
        "class": "form-control",
        "multiple": True
    }))

    class Meta:
        model = Logo
        fields = ['logo']
        widgets = {
            'logo': forms.FileField(),
        }


class GeneralInfoForm(forms.ModelForm):
    phone_number = PhoneNumberField()

    class Meta:
        model = GeneralInfo
        fields = ['company_name', 'phone_number', 'website', 'email']
        widgets = {
            'email': forms.EmailInput(),
        }





class AddressForm(forms.ModelForm):

    class Meta:
        model = Address
        fields = ['address_line_1', 'address_line_2', 'city', 'state', 'country', 'postal_code']
        widgets = {
            'country': CountrySelectWidget()
        }


class AdminSettingsForm(forms.ModelForm):

    class Meta:
        model = AdminSettings
        fields = ['timezone', 'language']



# Generated by Django 3.2.12 on 2023-02-04 09:52

from decimal import Decimal
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('subscriptions', '0014_subscription_created_date'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Invoice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateField(auto_now_add=True)),
                ('due_date', models.DateField()),
                ('invoice_number', models.CharField(max_length=100)),
                ('amount', models.DecimalField(decimal_places=2, max_digits=10)),
                ('credit_used', models.DecimalField(decimal_places=2, default=Decimal('0'), max_digits=10)),
                ('payment_method', models.CharField(max_length=50)),
                ('status', models.CharField(choices=[('paid', 'Paid'), ('unpaid', 'Unpaid'), ('pending', 'Pending'), ('cancelled', 'Cancelled'), ('refunded', 'Refunded')], max_length=10)),
                ('notes', models.TextField(blank=True)),
                ('pdf', models.FileField(upload_to='invoices/')),
                ('subscription', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='subscriptions.subscription')),
            ],
        ),
        migrations.CreateModel(
            name='Refund',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=10)),
                ('date', models.DateTimeField(auto_now_add=True)),
                ('reason', models.TextField()),
                ('invoice', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='billing.invoice')),
            ],
        ),
        migrations.CreateModel(
            name='Credit',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=10)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]
